/* popen.c: A "safe" pipe open routine.

Portions of this software are Copyright 1995 by Randall Atkinson and Dan
McDonald, All Rights Reserved. All Rights under this copyright are assigned
to the U.S. Naval Research Laboratory (NRL). The NRL Copyright Notice and
License Agreement applies to this software.

	History:

        Modified at NRL for OPIE 2.1. Optimized for only one pipe at a time.
                Added minimal version of sigprocmask(). Moved some pid_t
		dancing to the config headers.
	Modified at NRL for OPIE 2.0.
	Originally from BSD.

*/
/*
 * Copyright (c) 1988 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software written by Ken Arnold and
 * published in UNIX Review, Vol. 6, No. 8.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef lint
static char sccsid[] = "@(#)popen.c	5.9 (Berkeley) 2/25/91";
#endif	/* not lint */

#include "opie_cfg.h"

#include <sys/types.h>
#include <sys/wait.h>
#if HAVE_SIGNAL_H
#include <signal.h>
#endif /* HAVE_SIGNAL_H */
#if HAVE_SYS_SIGNAL_H
#include <sys/signal.h>
#endif /* HAVE_SYS_SIGNAL_H */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char **ftpglob __ARGS((register char *));
char **copyblk __ARGS((char **));
void blkfree __ARGS((char **));

#if !HAVE_SIGPROCMASK
#ifndef sigset_t
#define sigset_t int
#endif /* sigset_t */

int oursigprocmask(how, set, oset)
int how;
sigset_t *set;
sigset_t *oset;
{
	int old, new;

	if (set && (set != (sigset_t *)SIG_IGN) && (set != (sigset_t *)SIG_ERR))
		new = *set;
	else
		new = 0;

	switch(how) {
		case SIG_BLOCK:
			old = sigblock(new);
			if (oset && (oset != (sigset_t *)SIG_IGN) && (oset != (sigset_t *)SIG_ERR))
				*oset = old;
			return 0;

		case SIG_SETMASK:
                	old = sigsetmask(new);
			if (oset && (oset != (sigset_t *)SIG_IGN) && (oset != (sigset_t *)SIG_ERR))
				*oset = old;
			return 0;

		case SIG_UNBLOCK:
		default:
			return 0;
	}
}
#endif /* !HAVE_SIGPROCMASK */

/*
 * Special version of popen which avoids call to shell.  This insures noone
 * may create a pipe to a hidden program as a side effect of a list or dir
 * command.
 */
static pid_t child_pid = -1;
static int pipe_fd;

FILE *ftpd_popen(program, type)
char *program, *type;
{
  register char *cp;
  FILE *iop;
  int argc, gargc, pdes[2];
  char **pop, *argv[100], *gargv[1000], *vv[2];

  if (*type != 'r' && *type != 'w' || type[1])
    return (NULL);

  if (pipe(pdes) < 0)
    return (NULL);

  /* break up string into pieces */
  for (argc = 0, cp = program;; cp = NULL)
    if (!(argv[argc++] = strtok(cp, " \t\n")))
      break;

  /* glob each piece */
  gargv[0] = argv[0];
  for (gargc = argc = 1; argv[argc]; argc++) {
    if (!(pop = (char **) ftpglob(argv[argc]))) {
      /* globbing failed */
      vv[0] = argv[argc];
      vv[1] = NULL;
      pop = (char **) copyblk(vv);
    }
    argv[argc] = (char *) pop;	/* save to free later */
    while (*pop && gargc < 1000)
      gargv[gargc++] = *pop++;
  }
  gargv[gargc] = NULL;

  iop = NULL;
  switch (child_pid = vfork()) {
  case -1:	/* error */
    close(pdes[0]);
    close(pdes[1]);
    goto pfree;
    /* NOTREACHED */
  case 0:	/* child */
    if (*type == 'r') {
      if (pdes[1] != 1) {
	dup2(pdes[1], 1);
	dup2(pdes[1], 2);	/* stderr, too! */
	close(pdes[1]);
      }
      close(pdes[0]);
    } else {
      if (pdes[0] != 0) {
	dup2(pdes[0], 0);
	close(pdes[0]);
      }
      close(pdes[1]);
    }
    execv(gargv[0], gargv);
    _exit(1);
  }

  /* parent; assume fdopen can't fail...  */
  if (*type == 'r') {
    iop = fdopen(pipe_fd = pdes[0], type);
    close(pdes[1]);
  } else {
    iop = fdopen(pipe_fd = pdes[1], type);
    close(pdes[0]);
  }

pfree: for (argc = 1; argv[argc] != NULL; argc++) {
    blkfree((char **) argv[argc]);
    free((char *) argv[argc]);
  }
  return (iop);
}

#if !HAVE_SIGEMPTYSET && !defined(sigemptyset)
#define sigemptyset(x) memset(x, 0, sizeof(*x))
#endif /* !HAVE_SIGEMPTYSET */

#if !HAVE_SIGADDSET && !defined(sigaddset)
#define sigaddset(x, y) *x |= sigmask(y)
#endif /* !HAVE_SIGADDSET */

int ftpd_pclose(iop)
FILE *iop;
{
  register int fdes;

  int status;
  pid_t pid;
  sigset_t omask, mask;

  sigemptyset(&mask);
  sigaddset(&mask, SIGINT);
  sigaddset(&mask, SIGQUIT);
  sigaddset(&mask, SIGHUP);

  /* pclose returns -1 if stream is not associated with a `popened' command,
     or, if already `pclosed'. */
  if ((child_pid < 0) || (fileno(iop) != pipe_fd))
    return (-1);

  fclose(iop);
  sigprocmask(SIG_BLOCK, &mask, &omask);

  while ((pid = wait(&status)) != child_pid && (pid != -1));
  sigprocmask(SIG_SETMASK, &omask, NULL);

  child_pid = -1;
  pipe_fd = -1;

#ifdef WEXITSTATUS
  /* this is the fully POSIX compliant implementation */
  return (pid == -1 ? -1 : WEXITSTATUS(status));
#else
  return (pid == -1 ? -1 : status.w_status);
#endif
}
